"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var readline = require("readline-sync");
var mutate_op_1 = require("../enums/mutate-op");
var mutate_type_1 = require("../enums/mutate-type");
var index_1 = require("../index");
var createLetters = function (genome) {
    var letters = _.range(genome.options.length)
        .map(function (i) {
        return genome.g.letter();
    })
        .join("");
    return letters;
};
var gOptions = {
    loopGenes: true,
    genomeLength: 10,
    length: 10,
    geneLength: 50,
};
var pOptions = {
    initSize: 10,
    maxSize: 15,
    minSize: 5,
    mutateOptions: {
        mutateChance: 15,
        mutateOp: mutate_op_1.MutateOp.avg,
        sampleSize: 5,
        type: mutate_type_1.MutateType.safe,
    },
};
var pop = new index_1.ArtificialSelection(pOptions, gOptions, createLetters);
pop.genomes$
    .subscribe(function (genomes) {
    var genome = genomes[0];
    var letters = genomes
        .map(function (g) { return createLetters(g); });
    var gstr = letters
        .join(" ");
    var answer = readline.question("(1) keep\n(2) replace\n(3) reproduce\n(4) kill\n(5) random\n(6) generate\n");
    switch (answer) {
        case "keep":
        case "1":
            // requeues the current genome to the end of the array
            pop.keep();
            break;
        case "kill":
        case "2":
            // removes the current genome, adds a new offspring of the whole array to the end
            pop.kill();
            break;
        case "reproduce":
        case "3":
            // adds a new offspring of the whole array to the end
            pop.reproduce();
            break;
        case "delete":
        case "4":
            // removes the current genome
            pop.delete();
            break;
        case "random":
        case "5":
            // removes the current genome, adds a new random genome to the end
            pop.random();
            break;
        case "generate":
        case "6":
            // adds a new genome to the end
            pop.generate();
            break;
        default:
            // keep if input is invalid
            pop.keep();
            break;
    }
});
//# sourceMappingURL=letters.js.map