"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var fitness_objective_1 = require("../../enums/fitness-objective");
var mutate_op_1 = require("../../enums/mutate-op");
var avg_fitness_1 = require("../avg-fitness");
var sampled_mutate_many_1 = require("./sampled-mutate-many");
function safeSampledMutateMany(genomes, fitness, objective, mutateChance, mutateType, sampleSize) {
    if (objective === void 0) { objective = fitness_objective_1.FitnessObjective.maximize; }
    if (mutateChance === void 0) { mutateChance = 0.05; }
    if (mutateType === void 0) { mutateType = mutate_op_1.MutateOp.sub; }
    if (sampleSize === void 0) { sampleSize = 5; }
    var result = [];
    switch (objective) {
        case fitness_objective_1.FitnessObjective.maximize:
            while (result.length < genomes.length) {
                result = _.concat(result, sampled_mutate_many_1.sampledMutateMany(genomes, fitness, objective, mutateChance, mutateType, sampleSize)
                    .filter(function (g) { return fitness(g).fitness > avg_fitness_1.avgFitness(genomes, fitness); }));
            }
        case fitness_objective_1.FitnessObjective.minimize:
            while (result.length < genomes.length) {
                result = _.concat(result, sampled_mutate_many_1.sampledMutateMany(genomes, fitness, objective, mutateChance, mutateType, sampleSize)
                    .filter(function (g) { return fitness(g).fitness < avg_fitness_1.avgFitness(genomes, fitness); }));
            }
    }
    return result.slice(0, genomes.length);
}
exports.safeSampledMutateMany = safeSampledMutateMany;
//# sourceMappingURL=safe-sampled-mutate-many.js.map