"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var fitness_objective_1 = require("../../../enums/fitness-objective");
var avg_fitness_1 = require("../../avg-fitness");
var value_1 = require("../../value");
var reproduce_many_to_many_1 = require("./reproduce-many-to-many");
function safeReproduceManyToMany(genomes, n, fitness, objective, weights) {
    if (objective === void 0) { objective = fitness_objective_1.FitnessObjective.maximize; }
    if (weights === void 0) { weights = _.range(0, genomes.length).map(function (i) { return value_1.value(); }); }
    var result = [];
    var avgFit = avg_fitness_1.avgFitness(genomes, fitness);
    switch (objective) {
        case fitness_objective_1.FitnessObjective.maximize:
            while (result.length < n) {
                result = _.concat(result, reproduce_many_to_many_1.reproduceManyToMany(genomes, n, weights)
                    .filter(function (g) { return fitness(g).fitness > avgFit; }));
            }
            break;
        case fitness_objective_1.FitnessObjective.minimize:
            while (result.length < n) {
                result = _.concat(result, reproduce_many_to_many_1.reproduceManyToMany(genomes, n, weights)
                    .filter(function (g) { return fitness(g).fitness < avgFit; }));
            }
            break;
    }
    return result.slice(0, n);
}
exports.safeReproduceManyToMany = safeReproduceManyToMany;
//# sourceMappingURL=safe-reproduce-many-to-many.js.map