"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var fitness_objective_1 = require("../../../enums/fitness-objective");
var index_1 = require("../../../index");
var worst_1 = require("../../worst");
// produces many offspring from many genomes, each one selected from a sample
function sampledReproduceManyToMany(genomes, n, fitness, objective, sampleSize, weights) {
    if (objective === void 0) { objective = fitness_objective_1.FitnessObjective.maximize; }
    if (sampleSize === void 0) { sampleSize = 5; }
    if (weights === void 0) { weights = _.range(0, genomes.length).map(function (i) { return index_1.value(); }); }
    // create many genomes (according to n)
    return _.range(0, n)
        .map(function (i) {
        // create sample of genomes (according to sampleSize)
        var sample = _.range(0, sampleSize)
            .map(function (j) { return index_1.reproduceManyToOne(genomes, weights); });
        // return best genome from sample
        switch (objective) {
            case fitness_objective_1.FitnessObjective.maximize:
                return index_1.best(sample, fitness).genome;
            case fitness_objective_1.FitnessObjective.minimize:
                return worst_1.worst(sample, fitness).genome;
            default:
                return index_1.best(sample, fitness).genome;
        }
    });
}
exports.sampledReproduceManyToMany = sampledReproduceManyToMany;
//# sourceMappingURL=sampled-reproduce-many-to-many.js.map