"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var fitness_objective_1 = require("../../../enums/fitness-objective");
var best_1 = require("../../best");
var bottom_1 = require("../../bottom");
var top_1 = require("../../top");
var value_1 = require("../../value");
var worst_1 = require("../../worst");
var reproduce_many_to_one_1 = require("./reproduce-many-to-one");
// produce one offspring from many provided genomes,
// then returns the offspring if it is better than the top 50% on average, otherwise returns the best genome
function safeReproduceManyToOne(genomes, fitness, objective, weights) {
    if (objective === void 0) { objective = fitness_objective_1.FitnessObjective.maximize; }
    if (weights === void 0) { weights = _.range(0, genomes.length).map(function (i) { return value_1.value(); }); }
    var offspring = reproduce_many_to_one_1.reproduceManyToOne(genomes, weights);
    var offspringFitness = fitness(offspring).fitness;
    var t = top_1.top(genomes, fitness, 0.5);
    var b = bottom_1.bottom(genomes, fitness, 0.5);
    var topAvgFitness = _.meanBy(t, function (e) { return e.fitness; });
    var botAvgFitness = _.meanBy(b, function (e) { return e.fitness; });
    switch (objective) {
        case fitness_objective_1.FitnessObjective.maximize:
            if (offspringFitness > topAvgFitness) {
                return offspring;
            }
            else {
                return best_1.best(genomes, fitness).genome;
            }
        case fitness_objective_1.FitnessObjective.minimize:
            if (offspringFitness < botAvgFitness) {
                return offspring;
            }
            else {
                return worst_1.worst(genomes, fitness).genome;
            }
    }
}
exports.safeReproduceManyToOne = safeReproduceManyToOne;
//# sourceMappingURL=safe-reproduce-many-to-one.js.map