"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var fitness_objective_1 = require("../../../enums/fitness-objective");
var index_1 = require("../../../index");
var worst_1 = require("../../worst");
// produce one offspring from many provided genomes, each one selected from a sample
function sampledReproduceManyToOne(genomes, fitness, objective, weights, sampleSize) {
    if (objective === void 0) { objective = fitness_objective_1.FitnessObjective.maximize; }
    if (weights === void 0) { weights = _.range(0, genomes.length).map(function (i) { return index_1.value(); }); }
    if (sampleSize === void 0) { sampleSize = 5; }
    // produce offspring
    var offspring = _.range(0, sampleSize)
        .map(function (i) {
        return index_1.reproduceManyToOne(genomes, weights);
    });
    // return best genome
    switch (objective) {
        case fitness_objective_1.FitnessObjective.maximize:
            return index_1.best(offspring, fitness).genome;
        case fitness_objective_1.FitnessObjective.minimize:
            return worst_1.worst(offspring, fitness).genome;
    }
}
exports.sampledReproduceManyToOne = sampledReproduceManyToOne;
//# sourceMappingURL=sampled-reproduce-many-to-one.js.map