"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var genome_1 = require("../genotypes/genome");
var generate_genomes_1 = require("../operators/generate-genomes");
var mutate_1 = require("../operators/mutation/mutate");
var reproduce_many_to_one_1 = require("../operators/reproduction/many-to-one/reproduce-many-to-one");
var ArtificialPooledSelection = (function () {
    function ArtificialPooledSelection(popOptions, genOptions, create) {
        this.popOptions = popOptions;
        this.genOptions = genOptions;
        this.create = create;
        this.genomes = generate_genomes_1.generateGenomes(this.popOptions.initSize, this.genOptions);
        this.parents = generate_genomes_1.generateGenomes(this.popOptions.minParentPoolSize, this.genOptions)
            .map(function (g) { return ({ genome: g, age: 0 }); });
    }
    Object.defineProperty(ArtificialPooledSelection.prototype, "current", {
        get: function () {
            return this.genomes[0];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ArtificialPooledSelection.prototype, "currentResult", {
        get: function () {
            return this.create(this.current);
        },
        enumerable: true,
        configurable: true
    });
    // adds the current genome to parents
    ArtificialPooledSelection.prototype.keep = function () {
        var g = this.shift();
        this.addParent(g);
        this.ageParents();
    };
    // removes the current genome, adds a new offspring
    ArtificialPooledSelection.prototype.kill = function () {
        this.shift();
        this.ageParents();
    };
    // adds a new offspring
    ArtificialPooledSelection.prototype.reproduce = function () {
        var offspring = this.produceOffspring();
        if (this.genomes.length >= this.popOptions.maxSize) {
            this.shift(offspring);
        }
        this.add(offspring);
        this.ageParents();
    };
    // removes the current genome
    ArtificialPooledSelection.prototype.delete = function () {
        this.shift();
        this.ageParents();
    };
    // removes the current genome, adds a new random genome
    ArtificialPooledSelection.prototype.random = function () {
        if (this.genomes.length > this.popOptions.minSize) {
            this.add(new genome_1.Genome(this.genOptions));
        }
        else {
            this.shift(new genome_1.Genome(this.genOptions));
        }
        this.ageParents();
    };
    // adds a new random genome
    ArtificialPooledSelection.prototype.generate = function () {
        this.add(new genome_1.Genome(this.genOptions));
        this.ageParents();
    };
    ArtificialPooledSelection.prototype.produceOffspring = function () {
        var g = reproduce_many_to_one_1.reproduceManyToOne(this.parents.map(function (p) { return p.genome; }));
        g = mutate_1.mutate(g, this.popOptions.mutateOptions.mutateChance, this.popOptions.mutateOptions.mutateOp);
        return g;
    };
    ArtificialPooledSelection.prototype.ageParents = function (n) {
        if (n === void 0) { n = 1; }
        this.parents = this.parents.map(function (p) {
            return {
                age: p.age + n,
                genome: p.genome,
            };
        });
    };
    ArtificialPooledSelection.prototype.removeOldestParent = function () {
        this.removeParent(_.maxBy(this.parents, function (p) { return p.age; }));
    };
    ArtificialPooledSelection.prototype.add = function (genome) {
        if (this.genomes.length >= this.popOptions.maxSize) {
            this.shift();
        }
        this.genomes.push(genome);
    };
    ArtificialPooledSelection.prototype.addParent = function (genome) {
        if (this.parents.length >= this.popOptions.maxParentPoolSize) {
            this.removeOldestParent();
        }
        this.parents.push({ genome: genome, age: 0 });
    };
    ArtificialPooledSelection.prototype.shift = function (genome) {
        if (genome === void 0) { genome = null; }
        var g = this.genomes.shift();
        if (this.genomes.length < this.popOptions.minSize) {
            this.genomes.push(genome || this.produceOffspring());
        }
        return g;
    };
    ArtificialPooledSelection.prototype.remove = function (genome) {
        this.genomes = this.genomes.filter(function (g) { return !_.isEqual(g, genome); });
    };
    ArtificialPooledSelection.prototype.removeParent = function (parent) {
        this.parents = this.parents.filter(function (p) { return !_.isEqual(p, parent); });
    };
    return ArtificialPooledSelection;
}());
exports.ArtificialPooledSelection = ArtificialPooledSelection;
//# sourceMappingURL=artificial-pooled-selection.js.map