"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var rx_1 = require("rx");
var genome_1 = require("../genotypes/genome");
var generate_genomes_1 = require("../operators/generate-genomes");
var mutate_1 = require("../operators/mutation/mutate");
var replenish_many_1 = require("../operators/replenish-many");
var reproduce_many_to_one_1 = require("../operators/reproduction/many-to-one/reproduce-many-to-one");
var ArtificialSelection = (function () {
    function ArtificialSelection(popOptions, genOptions, create) {
        this.popOptions = popOptions;
        this.genOptions = genOptions;
        this.create = create;
        this.genomes = generate_genomes_1.generateGenomes(this.popOptions.initSize, this.genOptions);
        this.genomesSubject = new rx_1.BehaviorSubject(this.genomes);
    }
    Object.defineProperty(ArtificialSelection.prototype, "genomes$", {
        get: function () {
            return this.genomesSubject;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ArtificialSelection.prototype, "current", {
        get: function () {
            return this.genomes[0];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ArtificialSelection.prototype, "current$", {
        get: function () {
            return this.genomes$
                .map(function (genomes) { return genomes[0]; });
        },
        enumerable: true,
        configurable: true
    });
    // requeues the current genome to the end of the array
    ArtificialSelection.prototype.keep = function () {
        this.requeue();
        this.refresh();
    };
    // removes the current genome, adds a new offspring of the whole array to the end
    ArtificialSelection.prototype.kill = function () {
        this.dequeue();
        this.reproduce();
    };
    // adds a new offspring of the whole array to the end
    ArtificialSelection.prototype.reproduce = function (n) {
        var _this = this;
        if (n === void 0) { n = 1; }
        _.range(n).forEach(function (i) {
            var g = reproduce_many_to_one_1.reproduceManyToOne(_this.genomes);
            g = mutate_1.mutate(g, _this.popOptions.mutateOptions.mutateChance, _this.popOptions.mutateOptions.mutateOp);
            _this.add(g);
        });
        this.refresh();
    };
    ArtificialSelection.prototype.delete = function () {
        this.dequeue();
        this.refresh();
    };
    ArtificialSelection.prototype.random = function () {
        this.dequeue();
        this.add(new genome_1.Genome(this.genOptions));
        this.refresh();
    };
    ArtificialSelection.prototype.generate = function () {
        this.add(new genome_1.Genome(this.genOptions));
        this.refresh();
    };
    ArtificialSelection.prototype.refresh = function () {
        // console.log('refreshing...');
        this.genomesSubject.onNext(replenish_many_1.replenishMany(this.genomes));
    };
    ArtificialSelection.prototype.add = function (genome) {
        if (this.genomes.length >= this.popOptions.maxSize) {
            this.dequeue();
        }
        this.genomes.push(genome);
    };
    ArtificialSelection.prototype.addRange = function (genomes) {
        genomes.forEach(this.add);
    };
    ArtificialSelection.prototype.insert = function (genome, index, delCount) {
        this.genomes.splice(index, delCount, genome);
    };
    ArtificialSelection.prototype.dequeue = function () {
        var g = this.genomes.shift();
        if (this.genomes.length < this.popOptions.minSize) {
            this.reproduce();
        }
        return g;
    };
    ArtificialSelection.prototype.requeue = function () {
        this.add(this.dequeue());
    };
    ArtificialSelection.prototype.remove = function (genome) {
        this.genomes = this.genomes.filter(function (g) { return !_.isEqual(g, genome); });
    };
    ArtificialSelection.prototype.removeAtIndex = function (index) {
        var g = this.genomes[index];
        this.remove(g);
    };
    return ArtificialSelection;
}());
exports.ArtificialSelection = ArtificialSelection;
//# sourceMappingURL=artificial-selection.js.map