"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Chance = require("chance");
var d3 = require("d3-interpolate");
var chance = new Chance();
var Gene = (function () {
    function Gene(value) {
        this.value = value;
    }
    // returns a float, interpolated based on this.value
    Gene.prototype.float = function (min, max, value) {
        if (value === void 0) { value = this.value; }
        return d3.interpolateNumber(min, max)(value);
    };
    // returns an integer, interpolated based on this.value
    Gene.prototype.int = function (min, max, value) {
        if (value === void 0) { value = this.value; }
        return d3.interpolateRound(min, max)(value);
    };
    // returns a natural number, interpolated based on this.value
    Gene.prototype.natural = function (min, max, value) {
        if (value === void 0) { value = this.value; }
        return Math.max(0, this.int(min, max, value));
    };
    // returns a boolean, interpolated based on this.value
    Gene.prototype.bool = function (value) {
        if (value === void 0) { value = this.value; }
        return this.element([true, false], value);
    };
    // returns an upper or lower case letter, interpolated based on this.value
    Gene.prototype.letter = function (value) {
        if (value === void 0) { value = this.value; }
        var letters = "abcdefghijklmnopqrstuvwxyzABCDEFGIJKLMNOPQRSTUVWXYZ".split("");
        return this.element(letters, value);
    };
    // returns a lower case letter, interpolated based on this.value
    Gene.prototype.letterLower = function (value) {
        if (value === void 0) { value = this.value; }
        var letters = "abcdefghijklmnopqrstuvwxyz".split("");
        return this.element(letters, value);
    };
    // returns an upper case letter, interpolated based on this.value
    Gene.prototype.letterUpper = function (value) {
        if (value === void 0) { value = this.value; }
        var letters = "ABCDEFGIJKLMNOPQRSTUVWXYZ".split("");
        return this.element(letters, value);
    };
    // returns a character, interpolated based on this.value
    Gene.prototype.char = function (value) {
        if (value === void 0) { value = this.value; }
        var chars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!@#$%^&*()".split("");
        return this.element(chars, value);
    };
    // returns an element of the given array, interpolated based on this.value;
    Gene.prototype.element = function (array, value) {
        if (value === void 0) { value = this.value; }
        return array[this.int(0, array.length - 1, value)];
    };
    // returns a number of elements of the given array, the number of elements is interpolated based on this.value
    Gene.prototype.elements = function (array, value) {
        if (value === void 0) { value = this.value; }
        return array.slice(0, this.int(0, array.length, value));
    };
    // returns a number of randomly selected elements of the given array,
    // the number of elements is interpolated based on this.value
    Gene.prototype.randomElements = function (array, value) {
        if (value === void 0) { value = this.value; }
        return chance.shuffle(array).slice(0, this.int(0, array.length, value));
    };
    return Gene;
}());
exports.Gene = Gene;
//# sourceMappingURL=gene.js.map