"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Chance = require("chance");
var _ = require("lodash");
var index_1 = require("../index");
var chance = new Chance();
var Genome = (function () {
    function Genome(options, sequence, idLength) {
        if (sequence === void 0) { sequence = index_1.values(options.genomeLength * options.geneLength); }
        if (idLength === void 0) { idLength = Math.min(12, options.genomeLength); }
        this.options = options;
        this.sequence = sequence;
        this.idLength = idLength;
        this.genes = this.freshGenes;
    }
    Object.defineProperty(Genome.prototype, "id", {
        // returns a string of lower and upper case letters which are determined by the genome
        get: function () {
            var nucleos = this.freshGenes;
            var letters;
            var numLetters = nucleos.length / this.idLength;
            letters = _
                .chunk(nucleos, numLetters).slice(0, this.idLength) // group genes
                .map(function (n) {
                return n.reduce(function (a, b) { return new index_1.Gene((a.value + b.value) / 2); });
            })
                .map(function (n) { return n.letter(); }); // interpolate letter from averaged genees
            // return string of letters
            return letters.reduce(function (a, b) { return a + b; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Genome.prototype, "freshGenes", {
        // chunks values in sequence together into genes, determined by options.geneLength
        // the longer the genes are, the less sensitive to mutation they are.
        get: function () {
            var nucleos = _
                .chunk(this.sequence, this.options.geneLength)
                .map(function (n) { return _.reduce(n, function (memo, num) { return memo + num; }, 0) / n.length || 1; })
                .map(function (n) { return new index_1.Gene(n); });
            return nucleos;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Genome.prototype, "g", {
        // gets the next gene
        get: function () {
            if (this.genes.length === 0) {
                if (this.options.loopGenes) {
                    this.genes = this.freshGenes;
                }
                else {
                    throw new Error((this.id + " ran out of nucleos"));
                }
            }
            return this.genes.shift();
        },
        enumerable: true,
        configurable: true
    });
    // gets the next n genes
    Genome.prototype.gs = function (n) {
        var _this = this;
        if (this.genes.length < n) {
            if (this.options.loopGenes) {
                this.genes = this.freshGenes;
            }
            else {
                throw new Error((this.id + " ran out of nucleos"));
            }
        }
        return _.range(n).map(function (i) { return _this.g; });
    };
    return Genome;
}());
exports.Genome = Genome;
//# sourceMappingURL=genome.js.map